"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var visitor_1 = require("./visitor");
var introspection_visitor_1 = require("./introspection-visitor");
__export(require("./typescript-variables-to-object"));
exports.plugin = function (schema, documents, config) {
    var visitor = new visitor_1.TsVisitor(config);
    var printedSchema = graphql_1.printSchema(schema);
    var astNode = graphql_1.parse(printedSchema);
    var header = "type Maybe<T> = " + visitor.config.maybeValue + ";";
    var visitorResult = graphql_1.visit(astNode, { leave: visitor });
    var introspectionDefinitions = includeIntrospectionDefinitions(schema, documents, config);
    return [header].concat(visitorResult.definitions, introspectionDefinitions).join('\n');
};
function includeIntrospectionDefinitions(schema, documents, config) {
    var typeInfo = new graphql_1.TypeInfo(schema);
    var usedTypes = [];
    var documentsVisitor = graphql_1.visitWithTypeInfo(typeInfo, {
        Field: function () {
            var type = graphql_1.getNamedType(typeInfo.getType());
            if (graphql_1.isIntrospectionType(type) && !usedTypes.includes(type)) {
                usedTypes.push(type);
            }
        }
    });
    documents.forEach(function (doc) { return graphql_1.visit(doc.content, documentsVisitor); });
    var typesToInclude = [];
    usedTypes.forEach(function (type) {
        collectTypes(type);
    });
    var visitor = new introspection_visitor_1.TsIntrospectionVisitor(config, typesToInclude);
    var result = graphql_1.visit(graphql_1.parse(graphql_1.printIntrospectionSchema(schema)), { leave: visitor });
    // recursively go through each `usedTypes` and their children and collect all used types
    // we don't care about Interfaces, Unions and others, but Objects and Enums
    function collectTypes(type) {
        if (typesToInclude.includes(type)) {
            return;
        }
        typesToInclude.push(type);
        if (graphql_1.isObjectType(type)) {
            var fields_1 = type.getFields();
            Object.keys(fields_1).forEach(function (key) {
                var field = fields_1[key];
                var type = graphql_1.getNamedType(field.type);
                collectTypes(type);
            });
        }
    }
    return result.definitions;
}
//# sourceMappingURL=index.js.map